// src/create-external-subject.ts
function defaultShouldUpdate(a, b) {
  return !Object.is(a, b);
}
function createExternalSubject(options) {
  var _a;
  const shouldUpdate = (_a = options.shouldUpdate) != null ? _a : defaultShouldUpdate;
  let cache;
  const listeners = new Set();
  let request;
  const getCachedValue = () => {
    if (!cache) {
      cache = {
        value: options.read()
      };
    }
    return cache.value;
  };
  const requestUpdate = () => {
    if (shouldUpdate(getCachedValue(), options.read())) {
      if (request) {
        request.alive = false;
      }
      request = {
        promise: Promise.resolve().then(() => {
          if (request == null ? void 0 : request.alive) {
            request = void 0;
            if (cache) {
              cache.value = options.read();
            }
            listeners.forEach((listener) => {
              listener();
            });
          }
        }),
        alive: true
      };
    }
  };
  let unsubscribe;
  const subscribe = (handler) => {
    listeners.add(handler);
    if (options.lazySubscribe && options.subscribe && listeners.size === 1) {
      const unsub = options.subscribe(requestUpdate);
      if (unsub) {
        unsubscribe = unsub;
      }
    }
    return () => {
      listeners.delete(handler);
      if (options.lazySubscribe && unsubscribe) {
        unsubscribe();
      }
    };
  };
  if (!options.lazySubscribe && options.subscribe) {
    unsubscribe = options.subscribe(requestUpdate);
  }
  const destroy = () => {
    if (request) {
      request.alive = false;
    }
    listeners.clear();
    if (unsubscribe) {
      unsubscribe();
    }
  };
  return {
    subscribe,
    shouldUpdate,
    requestUpdate,
    getRequest: () => request,
    getCachedValue,
    getCurrentValue: options.read,
    destroy
  };
}

// src/use-external-subject.ts
import {useForceUpdate} from "@lyonph/react-hooks";
import {useDebugValue, useEffect} from "react";
function useExternalSubjectInternal(subject, suspense = false) {
  const forceUpdate = useForceUpdate();
  useEffect(() => {
    const unsubscribe = subject.subscribe(forceUpdate);
    return unsubscribe;
  }, [subject, forceUpdate]);
  useEffect(() => {
    if (!subject.getRequest()) {
      subject.requestUpdate();
      const ongoing2 = subject.getRequest();
      if (ongoing2) {
        forceUpdate();
      }
    }
  });
  const ongoing = subject.getRequest();
  if (ongoing) {
    if (suspense) {
      throw ongoing.promise;
    }
  } else {
    subject.requestUpdate();
    const current = subject.getRequest();
    if (current) {
      if (suspense) {
        throw current.promise;
      }
    }
  }
  return subject.getCachedValue();
}
function useExternalSubject(subject, suspense = false) {
  const value = useExternalSubjectInternal(subject, suspense);
  useDebugValue(value);
  return value;
}
export {
  createExternalSubject,
  useExternalSubject
};
//# sourceMappingURL=react-external-subject.esm.js.map
